@@ -1,6 +1,6 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-from trytond.model import ModelView, ModelSQL, fields
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from trytond.model import ModelView, ModelSQL, DeactivableMixin, fields, Unique
 from trytond.wizard import Wizard, StateView, Button
 from trytond.transaction import Transaction
 from trytond.pyson import Eval
@@ -11,21 +11,16 @@ __all__ = ['BOM', 'BOMInput', 'BOMOutput', 'BOMTree', 'OpenBOMTreeStart',
     'OpenBOMTreeTree', 'OpenBOMTree']
 
 
-class BOM(ModelSQL, ModelView):
+class BOM(DeactivableMixin, ModelSQL, ModelView):
     "Bill of Material"
     __name__ = 'production.bom'
 
     name = fields.Char('Name', required=True, translate=True)
-    active = fields.Boolean('Active', select=1)
     inputs = fields.One2Many('production.bom.input', 'bom', 'Inputs')
     outputs = fields.One2Many('production.bom.output', 'bom', 'Outputs')
     output_products = fields.Many2Many('production.bom.output',
         'bom', 'product', 'Output Products')
 
-    @staticmethod
-    def default_active():
-        return True
-
     def compute_factor(self, product, quantity, uom):
         '''
         Compute factor for an output product
@@ -45,14 +40,13 @@ class BOM(ModelSQL, ModelView):
             default = {}
         else:
             default = default.copy()
-        default['output_products'] = False
+        default.setdefault('output_products', None)
         return super(BOM, cls).copy(records, default=default)
 
 
 class BOMInput(ModelSQL, ModelView):
     "Bill of Material Input"
     __name__ = 'production.bom.input'
-    _rec_name = 'product'
 
     bom = fields.Many2One('production.bom', 'BOM', required=True,
         select=1, ondelete='CASCADE')
@@ -69,14 +63,19 @@ class BOMInput(ModelSQL, ModelView):
     unit_digits = fields.Function(fields.Integer('Unit Digits'),
         'on_change_with_unit_digits')
     quantity = fields.Float('Quantity', required=True,
+        domain=['OR',
+            ('quantity', '>=', 0),
+            ('quantity', '=', None),
+            ],
         digits=(16, Eval('unit_digits', 2)),
         depends=['unit_digits'])
 
     @classmethod
     def __setup__(cls):
         super(BOMInput, cls).__setup__()
+        t = cls.__table__()
         cls._sql_constraints = [
-            ('product_bom_uniq', 'UNIQUE(product, bom)',
+            ('product_bom_uniq', Unique(t, t.product, t.bom),
                 'product_bom_uniq'),
             ]
         cls._error_messages.update({
@@ -86,18 +85,14 @@ class BOMInput(ModelSQL, ModelView):
 
     @fields.depends('product', 'uom')
     def on_change_product(self):
-        res = {}
         if self.product:
-            uoms = self.product.default_uom.category.uoms
-            if (not self.uom or self.uom not in uoms):
-                res['uom'] = self.product.default_uom.id
-                res['uom.rec_name'] = self.product.default_uom.rec_name
-                res['unit_digits'] = self.product.default_uom.digits
+            category = self.product.default_uom.category
+            if not self.uom or self.uom.category != category:
+                self.uom = self.product.default_uom
+                self.unit_digits = self.product.default_uom.digits
         else:
-            res['uom'] = None
-            res['uom.rec_name'] = ''
-            res['unit_digits'] = 2
-        return res
+            self.uom = None
+            self.unit_digits = 2
 
     @fields.depends('product')
     def on_change_with_uom_category(self, name=None):
@@ -110,6 +105,13 @@ class BOMInput(ModelSQL, ModelView):
             return self.uom.digits
         return 2
 
+    def get_rec_name(self, name):
+        return self.product.rec_name
+
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        return [('product.rec_name',) + tuple(clause[1:])]
+
     @classmethod
     def validate(cls, boms):
         super(BOMInput, cls).validate(boms)
@@ -123,8 +125,7 @@ class BOMInput(ModelSQL, ModelView):
         self.product.check_bom_recursion()
 
     def compute_quantity(self, factor):
-        Uom = Pool().get('product.uom')
-        return Uom.round(self.quantity * factor, self.uom.rounding)
+        return self.uom.ceil(self.quantity * factor)
 
 
 class BOMOutput(BOMInput):
@@ -132,6 +133,18 @@ class BOMOutput(BOMInput):
     __name__ = 'production.bom.output'
     _table = 'production_bom_output'  # Needed to override BOMInput._table
 
+    def compute_quantity(self, factor):
+        return self.uom.floor(self.quantity * factor)
+
+    @classmethod
+    def delete(cls, outputs):
+        pool = Pool()
+        ProductBOM = pool.get('product.product-production.bom')
+        bom_products = [b for o in outputs for b in o.product.boms]
+        super(BOMOutput, cls).delete(outputs)
+        # Validate that output_products domain on bom is still valid
+        ProductBOM._validate(bom_products, ['bom'])
+
 
 class BOMTree(ModelView):
     'BOM Tree'
@@ -160,8 +173,10 @@ class BOMTree(ModelView):
             childs = cls.tree(input_.product, quantity, input_.uom)
             values = {
                 'product': input_.product.id,
+                'product.rec_name': input_.product.rec_name,
                 'quantity': quantity,
                 'uom': input_.uom.id,
+                'uom.rec_name': input_.uom.rec_name,
                 'unit_digits': input_.uom.digits,
                 'childs': childs,
             }
@@ -199,7 +214,8 @@ class OpenBOMTreeTree(ModelView):
     'Open BOM Tree'
     __name__ = 'production.bom.tree.open.tree'
 
-    bom_tree = fields.One2Many('production.bom.tree', None, 'BOM Tree')
+    bom_tree = fields.One2Many('production.bom.tree', None, 'BOM Tree',
+        readonly=True)
 
     @classmethod
     def tree(cls, bom, product, quantity, uom):
@@ -209,8 +225,10 @@ class OpenBOMTreeTree(ModelView):
         childs = Tree.tree(product, quantity, uom, bom=bom)
         bom_tree = [{
                 'product': product.id,
+                'product.rec_name': product.rec_name,
                 'quantity': quantity,
                 'uom': uom.id,
+                'uom.rec_name': uom.rec_name,
                 'unit_digits': uom.digits,
                 'childs': childs,
                 }]
@@ -226,11 +244,11 @@ class OpenBOMTree(Wizard):
     start = StateView('production.bom.tree.open.start',
         'production.bom_tree_open_start_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Ok', 'tree', 'tryton-ok', True),
+            Button('OK', 'tree', 'tryton-ok', True),
             ])
     tree = StateView('production.bom.tree.open.tree',
         'production.bom_tree_open_tree_view_form', [
-            Button('Change', 'start', 'tryton-go-previous'),
+            Button('Change', 'start', 'tryton-back'),
             Button('Close', 'end', 'tryton-close'),
             ])
 
